@@ -54,7 +54,6 @@ send_pinentry_environment (assuan_context_t ctx,
 gpg_error_t
 start_new_gpg_agent (assuan_context_t *r_ctx,
                      gpg_err_source_t errsource,
-                     const char *homedir,
                      const char *agent_program,
                      const char *opt_lc_ctype,
                      const char *opt_lc_messages,
@@ -68,12 +67,15 @@ start_new_gpg_agent (assuan_context_t *r_ctx,
 gpg_error_t
 start_new_dirmngr (assuan_context_t *r_ctx,
                    gpg_err_source_t errsource,
-                   const char *homedir,
                    const char *dirmngr_program,
                    int autostart, int verbose, int debug,
                    gpg_error_t (*status_cb)(ctrl_t, int, ...),
                    ctrl_t status_cb_arg);
 
+/* Return the version of a server using "GETINFO version".  */
+gpg_error_t get_assuan_server_version (assuan_context_t ctx,
+                                       int mode, char **r_version);
+
 
 /*-- asshelp2.c --*/
 
