@@ -1,5 +1,6 @@
 /* exectool.c - Utility functions to execute a helper tool
  * Copyright (C) 2015 Werner Koch
+ * Copyright (C) 2016 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -213,6 +214,8 @@ copy_buffer_init (struct copy_buffer *c)
 static void
 copy_buffer_shred (struct copy_buffer *c)
 {
+  if (c == NULL)
+    return;
   wipememory (c->buffer, sizeof c->buffer);
   c->writep = NULL;
   c->nread = ~0U;
@@ -303,10 +306,10 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
                         void *status_cb_value)
 {
   gpg_error_t err;
-  pid_t pid;
+  pid_t pid = (pid_t) -1;
   estream_t infp = NULL;
   estream_t extrafp = NULL;
-  estream_t outfp, errfp;
+  estream_t outfp = NULL, errfp = NULL;
   es_poll_t fds[4];
   int exceptclose[2];
   int extrapipe[2] = {-1, -1};
@@ -315,13 +318,34 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
   int argsaveidx;
   int count;
   read_and_log_buffer_t fderrstate;
-  struct copy_buffer cpbuf_in, cpbuf_out, cpbuf_extra; /* Fixme: malloc them. */
+  struct copy_buffer *cpbuf_in = NULL, *cpbuf_out = NULL, *cpbuf_extra = NULL;
 
   memset (fds, 0, sizeof fds);
   memset (&fderrstate, 0, sizeof fderrstate);
-  copy_buffer_init (&cpbuf_in);
-  copy_buffer_init (&cpbuf_out);
-  copy_buffer_init (&cpbuf_extra);
+
+  cpbuf_in = xtrymalloc (sizeof *cpbuf_in);
+  if (cpbuf_in == NULL)
+    {
+      err = my_error_from_syserror ();
+      goto leave;
+    }
+  copy_buffer_init (cpbuf_in);
+
+  cpbuf_out = xtrymalloc (sizeof *cpbuf_out);
+  if (cpbuf_out == NULL)
+    {
+      err = my_error_from_syserror ();
+      goto leave;
+    }
+  copy_buffer_init (cpbuf_out);
+
+  cpbuf_extra = xtrymalloc (sizeof *cpbuf_extra);
+  if (cpbuf_extra == NULL)
+    {
+      err = my_error_from_syserror ();
+      goto leave;
+    }
+  copy_buffer_init (cpbuf_extra);
 
   fderrstate.pgmname = pgmname;
   fderrstate.status_cb = status_cb;
@@ -329,7 +353,10 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
   fderrstate.buffer_size = 256;
   fderrstate.buffer = xtrymalloc (fderrstate.buffer_size);
   if (!fderrstate.buffer)
-    return my_error_from_syserror ();
+    {
+      err = my_error_from_syserror ();
+      goto leave;
+    }
 
   if (inextra)
     {
@@ -338,8 +365,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
         {
           log_error ("error running outbound pipe for extra fp: %s\n",
                      gpg_strerror (err));
-          xfree (fderrstate.buffer);
-          return err;
+          goto leave;
         }
       exceptclose[0] = extrapipe[0]; /* Do not close in child. */
       exceptclose[1] = -1;
@@ -369,9 +395,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
   if (err)
     {
       log_error ("error running '%s': %s\n", pgmname, gpg_strerror (err));
-      es_fclose (extrafp);
-      xfree (fderrstate.buffer);
-      return err;
+      goto leave;
     }
 
   fds[0].stream = infp;
@@ -407,7 +431,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
 
       if (fds[0].got_write)
         {
-          err = copy_buffer_do_copy (&cpbuf_in, input, fds[0].stream);
+          err = copy_buffer_do_copy (cpbuf_in, input, fds[0].stream);
           if (err)
             {
               log_error ("error feeding data to '%s': %s\n",
@@ -417,7 +441,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
 
           if (es_feof (input))
             {
-              err = copy_buffer_flush (&cpbuf_in, fds[0].stream);
+              err = copy_buffer_flush (cpbuf_in, fds[0].stream);
               if (err)
                 {
                   log_error ("error feeding data to '%s': %s\n",
@@ -433,7 +457,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
       if (fds[3].got_write)
         {
           log_assert (inextra);
-          err = copy_buffer_do_copy (&cpbuf_extra, inextra, fds[3].stream);
+          err = copy_buffer_do_copy (cpbuf_extra, inextra, fds[3].stream);
           if (err)
             {
               log_error ("error feeding data to '%s': %s\n",
@@ -443,7 +467,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
 
           if (es_feof (inextra))
             {
-              err = copy_buffer_flush (&cpbuf_extra, fds[3].stream);
+              err = copy_buffer_flush (cpbuf_extra, fds[3].stream);
               if (err)
                 {
                   log_error ("error feeding data to '%s': %s\n",
@@ -458,7 +482,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
 
       if (fds[1].got_read)
         {
-          err = copy_buffer_do_copy (&cpbuf_out, fds[1].stream, output);
+          err = copy_buffer_do_copy (cpbuf_out, fds[1].stream, output);
           if (err)
             {
               log_error ("error reading data from '%s': %s\n",
@@ -468,7 +492,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
 
           if (es_feof (fds[1].stream))
             {
-              err = copy_buffer_flush (&cpbuf_out, output);
+              err = copy_buffer_flush (cpbuf_out, output);
               if (err)
                 {
                   log_error ("error reading data from '%s': %s\n",
@@ -494,7 +518,7 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
   pid = (pid_t)(-1);
 
  leave:
-  if (err)
+  if (err && pid != (pid_t) -1)
     gnupg_kill_process (pid);
 
   es_fclose (infp);
@@ -505,10 +529,12 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
     gnupg_wait_process (pgmname, pid, 1, NULL);
   gnupg_release_process (pid);
 
-  copy_buffer_shred (&cpbuf_in);
-  copy_buffer_shred (&cpbuf_out);
-  if (inextra)
-    copy_buffer_shred (&cpbuf_extra);
+  copy_buffer_shred (cpbuf_in);
+  xfree (cpbuf_in);
+  copy_buffer_shred (cpbuf_out);
+  xfree (cpbuf_out);
+  copy_buffer_shred (cpbuf_extra);
+  xfree (cpbuf_extra);
   xfree (fderrstate.buffer);
   return err;
 }
