@@ -96,6 +96,7 @@ enum
     STATUS_SIG_CREATED,
     STATUS_SESSION_KEY,
     STATUS_NOTATION_NAME,
+    STATUS_NOTATION_FLAGS,
     STATUS_NOTATION_DATA,
     STATUS_POLICY_URL,
     STATUS_KEY_CREATED,
@@ -105,6 +106,7 @@ enum
     STATUS_INV_SGNR,
     STATUS_NO_RECP,
     STATUS_NO_SGNR,
+    STATUS_KEY_CONSIDERED,
 
     STATUS_ALREADY_SIGNED,
     STATUS_KEYEXPIRED,
@@ -140,6 +142,7 @@ enum
 
     STATUS_TRUNCATED,
     STATUS_MOUNTPOINT,
+    STATUS_BLOCKDEV,
 
     STATUS_PINENTRY_LAUNCHED,
 
