@@ -35,18 +35,18 @@ print_open_fds (int *array)
 {
   int n;
 
+  if (!verbose)
+    return;
+
   for (n=0; array[n] != -1; n++)
     ;
   printf ("open file descriptors: %d", n);
-  if (verbose)
-    {
-      putchar (' ');
-      putchar (' ');
-      putchar ('(');
-      for (n=0; array[n] != -1; n++)
-        printf ("%d%s", array[n], array[n+1] == -1?"":" ");
-      putchar (')');
-    }
+  putchar (' ');
+  putchar (' ');
+  putchar ('(');
+  for (n=0; array[n] != -1; n++)
+    printf ("%d%s", array[n], array[n+1] == -1?"":" ");
+  putchar (')');
   putchar ('\n');
 }
 
@@ -84,7 +84,8 @@ test_close_all_fds (void)
   system (buffer);
 #endif
 
-  printf ("max. file descriptors: %d\n", max_fd);
+  if (verbose)
+    printf ("max. file descriptors: %d\n", max_fd);
   array = xget_all_open_fds ();
   print_open_fds (array);
   for (initial_count=n=0; array[n] != -1; n++)
@@ -149,7 +150,7 @@ test_close_all_fds (void)
             print_open_fds (array);
           free (array);
         }
-      
+
       /* Check whether the except list works.  */
       close_all_fds (3, except);
       array = xget_all_open_fds ();
@@ -180,9 +181,8 @@ main (int argc, char **argv)
       verbose = 1;
       argc--; argv++;
     }
-  
+
   test_close_all_fds ();
 
   return 0;
 }
-
